﻿using System;
using System.ComponentModel;
using System.Drawing;
using System.Runtime.CompilerServices;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Problems.DataAnalysis.Annotations;
using HEAL.Attic;

namespace HeuristicLab.Problems.DataAnalysis {
  [StorableType("8109BE58-CCFB-4462-A2F4-EEE5DFADAFF7")]
  [Item("Interval Constraint", "Constraint on intervals.")]
  public class IntervalConstraint : NamedItem {
    private string expression;
    public string Expression {
      get => expression;
      set {
        if (value != expression) {
          expression = value;
          OnChanged(EventArgs.Empty);
        }
      }
    }
    public string Definition { get; set; }
    public Interval Interval { get; set; }
    private bool inclusiveLowerBound;
    public bool InclusiveLowerBound {
      get => inclusiveLowerBound;
      set {
        if (value != inclusiveLowerBound) {
          inclusiveLowerBound = value;
          OnChanged(EventArgs.Empty);
        }
      }
    }
    public bool InclusiveUpperBound { get; set; }
    public bool IsDerivation { get; set; }

    private string variable;
    public string Variable {
      get => variable;
      set {
        if (value != variable) {
          variable = value;
          OnChanged(EventArgs.Empty);
        }
      }
    }

    public int numberOfDerivation;
    public int NumberOfDerivation {
      get => numberOfDerivation;
      set {
        if (value != numberOfDerivation) {
          numberOfDerivation = value;
          OnChanged(EventArgs.Empty);
        }
      }
    }

    private bool enabled;
    public bool Enabled {
      get => enabled;
      set {
        if (value != enabled) {
          enabled = value;
          OnChanged(EventArgs.Empty);
        }
      }
    }

    public IntervalConstraint() {
      name = "Empty";
      expression = "Empty";
      Interval = new Interval(0, 0);
      inclusiveLowerBound = true;
      inclusiveLowerBound = true;
      IsDerivation = false;
      enabled = false;
    }

    public IntervalConstraint(string name) {
      base.name = name;
      enabled = true;
    }

    public IntervalConstraint(string expression, string definition, Interval interval, bool inclusiveLowerBound,
      bool inclusiveUpperBound, bool isDerivation, string variable, int numberOfDerivation, bool isChecked) {
      base.name = expression;
      Expression = expression;
      Definition = definition;
      Interval = interval;
      InclusiveLowerBound = inclusiveLowerBound;
      InclusiveUpperBound = inclusiveUpperBound;
      IsDerivation = isDerivation;
      Variable = variable;
      NumberOfDerivation = numberOfDerivation;
      enabled = isChecked;
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new IntervalConstraint(this, cloner);
    }

    protected IntervalConstraint(IntervalConstraint original, Cloner cloner)
      : base(original, cloner) {
      this.Expression = original.Expression;
      this.Definition = original.Definition;
      this.Interval = original.Interval;
      this.InclusiveLowerBound = original.InclusiveLowerBound;
      this.InclusiveUpperBound = original.InclusiveUpperBound;
      this.IsDerivation = original.IsDerivation;
      this.Variable = original.Variable;
      this.NumberOfDerivation = original.NumberOfDerivation;
      this.enabled = original.enabled;
    }

    public event EventHandler Changed;
    protected virtual void OnChanged(EventArgs e) {
      EventHandler handlers = Changed;
      if (handlers != null)
        handlers(this, e);
    }
  }
}
